#include"animationedit.h"

#include"animationedit_new.h"

#include<QMessageBox>
#include<QCloseEvent>
#include<QSettings>

#include"../core/mfilename.h"
#include"../core/mframeanimation.h"

#include"frameanimationedit.h"

#include"viewanimation.h"

#include"addframe.h"

AnimationEdit::AnimationEdit(QWidget* parent)
	:QMainWindow(parent){
	ui.setupUi(this);

	changed = false;
	what = new MFrameAnimation();
	whatFileName = MFilename("",MFilename::Invalid);

	FrameAnimationEdit* e = new FrameAnimationEdit(what, this);
	setCentralWidget(e);
	connect(this, SIGNAL(addFrame(MFilename, int, int)), e, SLOT(addFrame(MFilename, int, int)));
}

void AnimationEdit::switchTo(MImage* mi){
	if(mi == NULL)
		return;
	MImage* w = what;

	what = mi;

  switch(what->getType()){
	case 0:{	//scripted

	}break;
	case 1:{	//frame
//		action_Add->setEnabled(true);

		FrameAnimationEdit* e = new FrameAnimationEdit(what, this);
		setCentralWidget(e);
		connect(this, SIGNAL(addFrame(MFilename, int, int)), e, SLOT(addFrame(MFilename, int, int)));
	}break;
	case 2:{	//static

	}break;
  }

	delete w;
}

void AnimationEdit::on_action_Add_activated(){
	AddFrame af;

	if(af.exec() == QDialog::Accepted){
		MFilename a(af.getImageName(), MFilename::StaticImage, this);

		if(a.isValid()){
			emit addFrame(a, af.getMinimumTime(), af.getMaximumTime());
			changed = true;
		}
	}
}

void AnimationEdit::closeEvent(QCloseEvent* e){
	if(changed)
		switch(
			QMessageBox::question(this, 
				tr("Confirm exit"), 
				tr("Save before exiting?"),
				 QMessageBox::Save, QMessageBox::Discard, QMessageBox::Cancel)
		){
			case QMessageBox::Save:
				on_action_Save_activated();
			break;
			case QMessageBox::Discard:
				e->accept();
				return;
			case QMessageBox::Cancel:
				e->ignore();
				return;
		}
	else
		e->accept();
}

void AnimationEdit::on_action_Open_activated(){
	MFilename fileToOpen = MFilename::getOpenFileName(MFilename::Image, this);

	MImage* a = MImage::load(fileToOpen);

	whatFileName = fileToOpen.forDisk();
	switchTo(a);
}

void AnimationEdit::on_action_View_activated(){
	ViewAnimation v(what, this);
	v.exec();
}

void AnimationEdit::on_action_Save_activated(){
	if(!whatFileName.isValid())
		on_actionSave_As_activated();
	else{
		what->save(whatFileName);
	}
}

void AnimationEdit::on_actionSave_As_activated(){
	MFilename whatName = MFilename::getSaveFileName(MFilename::Image, this);

	if(whatName.isValid()){
		whatFileName = whatName;
		what->save(whatFileName);
	}
}

void AnimationEdit::on_action_New_activated(){
	if(changed)
		switch(
			QMessageBox::question(this, 
				tr("Confirm new"), 
				tr("Save changes before creating new?"),
				 QMessageBox::Save, QMessageBox::Discard, QMessageBox::Cancel)
		){
			case QMessageBox::Save:
				on_action_Save_activated();
			break;
			case QMessageBox::Discard:
				// nothing intended
			break;
			case QMessageBox::Cancel:
				return;
		}

	AnimationEdit_New test;
	if(test.exec() == QDialog::Accepted){
		MImage* a;
		changed = false;
		whatFileName = MFilename("",MFilename::Invalid);
		// create a new one
		a = new MFrameAnimation();

		switchTo(a);
	}
}
