#include"frameanimationedit.h"

FrameAnimationEdit::FrameAnimationEdit(MImage* b,QWidget* parent)
	: QWidget(parent){
	buffer = dynamic_cast<MFrameAnimation*>(b);
	invalidate_cache();
	dragging = -1;
	selected = -1;

	editAct = new QAction(tr("&Edit"), this);
	connect(editAct, SIGNAL(triggered()), this, SLOT(editFrame()));

	removeAct = new QAction(tr("&Remove"), this);
	connect(removeAct, SIGNAL(triggered()), this, SLOT(removeFrame()));
}

void FrameAnimationEdit::editFrame(){

}

void FrameAnimationEdit::removeFrame(){

}

void FrameAnimationEdit::paintEvent(QPaintEvent* event){
	QPainter painter(this);

	int xm = width() / 80;
	int ym = height() / 80;

	int x=0,y=0;
	int xp = 16;
	int yp = 16;

	for(int a=0;a<buffer->frames.count();a++){

		if(a == selected){
			QBrush brush(Qt::yellow,Qt::Dense5Pattern);
			painter.setBrush(brush);
			painter.drawRect(xp-8, yp-8, 80, 80);
		}


		painter.drawPixmap(xp,yp,cache.at(a));

		x++;
		xp += 80;
		if(x == xm){
			x = 0;
			y++;
			xp = 16;
			yp += 80;
		}
		if(y == ym)
			break;	//no room left in the window, so just break out
	}

	painter.end();

	event->accept();
}

int FrameAnimationEdit::frameat(const QPoint& p){
	int x = (p.x()-16)/80;
	int y = (p.y()-16)/80;

	int xm = width() / 80;
	int ym = height() / 80;

	if(x >= xm || y>= ym)
		return -1;

	int n = y*xm + x;

	if(n>=buffer->frames.count())
		return -1;
	else
		return n;

	return 0;	//never reached
}

void FrameAnimationEdit::invalidate_cache(){
	cache.clear();
	for(int a=0;a<buffer->frames.count();a++){
		QPixmap pix = QPixmap::fromImage(buffer->frames[a].getImage());
		cache.append(pix.scaled(64,64,Qt::KeepAspectRatio));
	}
}

void FrameAnimationEdit::mouseMoveEvent(QMouseEvent* e){
	if(dragging == -1){
		e->ignore();
		return;
	}else{
		int f = frameat(e->pos());
		if(f != -1 && f != dragging){
			selected = f;
			update();
		}
		e->accept();
	}
}

void FrameAnimationEdit::mousePressEvent(QMouseEvent* e){
	int i = frameat(e->pos());
	dragging = i;
	selected = i;
	e->accept();
	update();
}

void FrameAnimationEdit::mouseReleaseEvent(QMouseEvent* e){
	if(dragging == -1){
		e->ignore();
		return;
	}else{
		int f = frameat(e->pos());
		if(f != -1 && f != dragging){
			moveframe(dragging,f);
			selected = f;
			dragging = -1;
			update();
		}
		e->accept();
	}
}

void FrameAnimationEdit::mouseDoubleClickEvent(QMouseEvent* e){
	int n = frameat(e->pos());
	if(n == -1)
		e->ignore();
	else{
		editFrame();
		e->accept();
	}
}

void FrameAnimationEdit::contextMenuEvent(QContextMenuEvent* e){
	int f = frameat(e->pos());
	if(f != -1){
		QMenu menu(this);
		menu.addAction(editAct);
		menu.addAction(removeAct);
		menu.exec(e->globalPos());
		e->accept();
	}
	e->ignore();
}

void FrameAnimationEdit::moveframe(int f,int w){
	buffer->frames.swap(f,w);
	invalidate_cache();
}

void FrameAnimationEdit::addFrame(MFilename name, int lt, int ht){
	MFrame f(name, lt, ht);
	buffer->frames.append(f);

	if(buffer->height < f.getHeight())
		buffer->height = f.getHeight();

	if(buffer->width < f.getWidth())
		buffer->width = f.getWidth();

	invalidate_cache();
}

