#include<QtGui>

#include"../core/mframeanimation.h"

/*
	This is the central widget of the animationedit main window.

	It is responsible for:
		Drawing the frames.
		Deciding when a new frame has been selected.
		Emitting a signal saying when a new frame has been selected.
		Emitting context menu requested signals, with frame number given.
		Handling keyboard input for frame manipulation.
		Handling the drag and drop.
		Emitting a signal saying a frame has moved.

	It requires a pointer to the animation buffer, from the main window widget.
	It implements this with a pointer to the pointer, passed in the constructor.
	This pointer must be valid, or bad things will happen.
*/

class FrameAnimationEdit : public QWidget{
	Q_OBJECT
  public:
	FrameAnimationEdit(MImage* b,QWidget* parent = 0);
	int selected;	// the currently selected frame
  private:
	void paintEvent(QPaintEvent* event);
	void mousePressEvent(QMouseEvent* e);
	void mouseReleaseEvent(QMouseEvent* e);
	void mouseDoubleClickEvent(QMouseEvent* e);
	void mouseMoveEvent(QMouseEvent* e);
	void contextMenuEvent(QContextMenuEvent* e);

	int frameat(const QPoint& p);

	QList<QPixmap> cache;

	MFrameAnimation* buffer;

	QAction* editAct;
	QAction* removeAct;

	int dragging;	// the frame number we are currently dragging, or -1 if not dragging
	void moveframe(int f,int w);	// emitted when the user has drag and dropped a frame
					// f is frame number, w is where he dropped it
	void invalidate_cache();	// this should be called to force this widget to reread the animation and update it's cached thumbnails

  public slots:
	void addFrame(MFilename name, int lt, int ht);
	void editFrame();
	void removeFrame();
};

