#include"mfilename.h"

#include"functions.h"

#include<QFileDialog>

#include<QStringList>
#include<QDir>
#include<QObject>
#include<QMessageBox>
#include<QFileInfo>

#include<QSettings>

#include<QInputDialog>

/*
To get the save name, take the base path (defined as HOME/arsd/m3/creator/type/) and strip it, leaving
subdir and filename.

To take that to a load name, just stick the base path back on, determining type from the extension.

To take a HDD name to a M name, strip off the path and move it to the base path place.

*/

MFilename::MFilename(){}

MFilename::MFilename(QString filename, Type t, QWidget* parent){
	if(t != Invalid)
		fromFilename(filename, t, parent);
	else
		type = t;
}

bool MFilename::isValid(){
	return type != Invalid;
}

void MFilename::fromFilename(QString& fn, Type t, QWidget* parent){
	if(fn.isEmpty())
		return;

	QFileInfo fi(fn);

	if(fi.baseName().isEmpty())
		return;

	QString path = fi.absolutePath();
	QString rightPath = correctBaseFolder() + correctSubfolder(t);

	QDir d(rightPath);
	if(!d.exists())
		d.mkpath(rightPath);

	QString rightName = rightPath + fi.baseName() + "." + fi.completeSuffix();

	path += "/";

	if(!path.contains(rightPath)){
		QFile f(fn);
		QFile fr(rightName);

		if(fr.exists()){
			if(QMessageBox::question(parent, QObject::tr("Select file version"),
				QObject::tr("A file with the same name already exists in your loadable folder.\n\nOverwrite the existing file with the new one?"),
				QMessageBox::Yes,
				QMessageBox::No
			) == QMessageBox::Yes){
				fr.remove();
				f.copy(rightName);
			}
		}
		else
			f.copy(rightName);
	}

	name = fi.baseName() + "." + fi.completeSuffix();
	type = t;
}

MFilename::MFilename(QString fromM){
	name = fromM;
	type = Image;	//FIXME
}

QString MFilename::correctBaseFolder(){
	QString path = QDir::homePath();
	path += "/arsd/m3/";
	path += copyCreatorName() + "/";
	return path;
}

QString MFilename::correctSubfolder(Type t){
	QString path;
	switch(t){
		case Invalid:
			// logic error, should never be called
			return QString();
		break;
		case StaticImage:
		case Image:
			path = "images/";
		break;
		case Sound:
			path = "sfx/";
		break;
		case Music:
			path = "bgm/";
		break;
		case Map:
			path = "maps/";
		break;
		case Tile:
			path = "tiles/";
		break;
	}


	return path;
}

MFilename MFilename::getOpenFileName(Type t, QWidget* parent){
	QString types;
	QString greeting;
	QStringList extensions;
	switch(t){
		case Invalid:
			QMessageBox::warning(parent,"Bug","You should never see this. Post what you did to cause this to happen so I can fix it.");
			return MFilename(QString(), Invalid);
			// logic error, should never be called
		break;
		case StaticImage:
			types = QObject::tr("Images (*.png)");
			greeting = QObject::tr("Open Image");
			extensions << "png";
		break;
		case Image:
			types = QObject::tr("All supported types (*.png *.m3a);;Animations (*.m3a);;Pictures (*.png)");
			greeting = QObject::tr("Open graphic");
			extensions << "png" << "m3a";
		break;
		case Sound:
			types = QObject::tr("Wave files (*.wav)");
			greeting = QObject::tr("Open sound effect");
			extensions << "wav";

		break;
		case Music:
			types = QObject::tr("All supported types (*.mid *.ogg);;MIDI files (*.mid);;Ogg Vorbis audio (*.ogg)");
			greeting = QObject::tr("Open music");
			extensions << "mid" << "ogg";

		break;
		case Map:
			types = QObject::tr("Maps (*.m3m)");
			greeting = QObject::tr("Open map");
			extensions << "m3m";

		break;
		case Tile:
			types = QObject::tr("Tiles (*.m3t)");
			greeting = QObject::tr("Open tile");
			extensions << "m3t";
		break;
	}

	QSettings s;

	QString d = s.value("Last open dir", correctBaseFolder() + correctSubfolder(t)).toString();

	QString fileName = QFileDialog::getOpenFileName(parent, greeting,
				d,
				types);

	if(fileName.isEmpty())
		return MFilename("", Invalid);
	else{
		QFileInfo fi(fileName);
		s.setValue("Last open dir", fi.absolutePath());
		return MFilename(fileName, t);
	}
}

MFilename MFilename::getSaveFileName(Type t, QWidget* parent){
//FIXME: does not confirm overwrite
	QString name = QInputDialog::getText(parent, "Save file", "Filename (omit extension and folder):");
	switch(t){
		case Invalid:
			QMessageBox::warning(parent,"Bug","You should never see this. Post what you did to cause this to happen so I can fix it.");
			return MFilename(name);
			// logic error, should never be called
		break;
		case StaticImage:
			name += ".png";
		break;
		case Image:
			name += ".m3a";
		break;
		case Sound:
			name += ".wav";
		break;
		case Music:
			name += ".mid";
		break;
		case Map:
			name += ".m3m";
		break;
		case Tile:
			name += ".m3t";
		break;
	}

	MFilename mf(name);
	mf.type = t;
	return mf;
}

QString MFilename::forDisk(){
	return correctBaseFolder() + correctSubfolder(type) + name;
}

QString MFilename::forStoringInFile(){
	return name;
}

