#ifndef MFILENAME_H
#define MFILENAME_H

#include<QString>

class QWidget;

/*
	Necessary operations for the MFilename.

	Input:
		arbitrary filename for loading				(from QFileDialog::getOpenFileName())
		M filename for conversion to arbitrary filename		(from a saved M3 file)
	Output:
		M filename for storing within files			(for putting in a M3 file, should be converted when loading)
		arbitrary filename for saving				(for QFile::open())
*/

/*
	Filenames as they are stored:

		:folder/file.ext if from ARSD
		folder/file.ext  if not.

	Creator is determined by the current creator name.
	Right folder is determined by the file extension.
*/

/*
	File extensions to file types:

	.png		Static images
	.m3a		MImage
	.wav		Sound
	.ogg .mid	Music
	.m3m		Map
	.m3t		Tile
*/

class MFilename{
  public:
	enum Type {Invalid = -1, StaticImage, Image, Sound, Music, Map, Tile };

	MFilename(QString filename, Type t, QWidget* parent = 0);	// constructs one from a regular filename, used when loading
						// SHOULD NEVER BE CALLED FROM ENGINE
	MFilename(QString fromM);	// just copies the string in, verifying it is valid
	MFilename();

	static MFilename getOpenFileName(Type t, QWidget* parent = 0);
	static MFilename getSaveFileName(Type t, QWidget* parent = 0);

	static QString correctBaseFolder();
	static QString correctSubfolder(Type t);

	QString forDisk();
	QString forStoringInFile();

	bool isValid();

  private:
	Type type;
	QString name;

	void fromFilename(QString& fn, Type t, QWidget* parent);

//	QString folder;
//	QString fsName;
//	static MFilename fromFilename(QString& filename);
//	static MFilename toFilename(QString& filename);
};

#endif

