#include"tileedit.h"

#include<QMessageBox>
#include<QCloseEvent>

TileEdit::TileEdit(QWidget* parent)
	: QMainWindow(parent){
	ui.setupUi(this);

	whatFileName = MFilename("", MFilename::Invalid);
	changed = false;

	what = new MTile();
}

void TileEdit::commit(){
	what->overdraw = ui.overdrawBox->isChecked();
	if(ui.noDamageButton->isChecked())
		what->damage = 0;
	if(ui.lowDamageButton->isChecked())
		what->damage = 1;
	if(ui.highDamageButton->isChecked())
		what->damage = 2;


	what->animationNames[0] = MFilename(ui.animationFile1->text(), MFilename::Image, this);
	what->animationNames[1] = MFilename(ui.animationFile2->text(), MFilename::Image, this);
	what->animationNames[2] = MFilename(ui.animationFile3->text(), MFilename::Image, this);
	what->animationNames[3] = MFilename(ui.animationFile4->text(), MFilename::Image, this);
	what->animationNames[4] = MFilename(ui.animationFile5->text(), MFilename::Image, this);
	what->animationNames[5] = MFilename(ui.animationFile6->text(), MFilename::Image, this);

	what->canWalk = ui.canWalk->isChecked();

	what->canMove[0] = ui.canMove1->isChecked();
	what->canMove[1] = ui.canMove2->isChecked();
	what->canMove[2] = ui.canMove3->isChecked();
	what->canMove[3] = ui.canMove4->isChecked();
	what->canMove[4] = ui.canMove5->isChecked();
	what->canMove[5] = ui.canMove6->isChecked();

	what->canPark[0] = ui.canPark1->isChecked();
	what->canPark[1] = ui.canPark2->isChecked();
	what->canPark[2] = ui.canPark3->isChecked();
	what->canPark[3] = ui.canPark4->isChecked();
	what->canPark[4] = ui.canPark5->isChecked();
	what->canPark[5] = ui.canPark6->isChecked();

	what->battleFrequency = ui.battleFrequency->value();
	what->battleBackground = MFilename(ui.battleBackground->text(), MFilename::Image, this);
}

void TileEdit::revert(){
	ui.overdrawBox->setChecked(what->overdraw);
	ui.noDamageButton->setChecked(what->damage == 0);
	ui.lowDamageButton->setChecked(what->damage == 1);
	ui.highDamageButton->setChecked(what->damage == 2);


	ui.animationFile1->setText(what->animationNames[0].forDisk());
	ui.animationFile2->setText(what->animationNames[1].forDisk());
	ui.animationFile3->setText(what->animationNames[2].forDisk());
	ui.animationFile4->setText(what->animationNames[3].forDisk());
	ui.animationFile5->setText(what->animationNames[4].forDisk());
	ui.animationFile6->setText(what->animationNames[5].forDisk());

	ui.canWalk->setChecked(what->canWalk);

	ui.canMove1->setChecked(what->canMove[0]);
	ui.canMove2->setChecked(what->canMove[1]);
	ui.canMove3->setChecked(what->canMove[2]);
	ui.canMove4->setChecked(what->canMove[3]);
	ui.canMove5->setChecked(what->canMove[4]);
	ui.canMove6->setChecked(what->canMove[5]);

	ui.canPark1->setChecked(what->canPark[0]);
	ui.canPark2->setChecked(what->canPark[1]);
	ui.canPark3->setChecked(what->canPark[2]);
	ui.canPark4->setChecked(what->canPark[3]);
	ui.canPark5->setChecked(what->canPark[4]);
	ui.canPark6->setChecked(what->canPark[5]);

	ui.battleFrequency->setValue(what->battleFrequency);
	ui.battleBackground->setText(what->battleBackground.forDisk());
}

void TileEdit::closeEvent(QCloseEvent* e){
	if(changed)
		switch(
			QMessageBox::question(this, 
				tr("Confirm exit"), 
				tr("Save before exiting?"),
				 QMessageBox::Save, QMessageBox::Discard, QMessageBox::Cancel)
		){
			case QMessageBox::Save:
				on_action_Save_activated();
			break;
			case QMessageBox::Discard:
				e->accept();
				return;
			case QMessageBox::Cancel:
				e->ignore();
				return;
		}
	else
		e->accept();
}

void TileEdit::on_action_New_activated(){
	delete what;
	what = new MTile();
//	revert();
	whatFileName = MFilename("", MFilename::Invalid);
	//FIXME: It will display the same stuff, this should probably be set back to default
}

void TileEdit::on_action_Save_activated(){
	if(!whatFileName.isValid())
		on_actionSave_As_activated();
	else{
		commit();
		what->save(whatFileName);
	}
}

void TileEdit::on_actionSave_As_activated(){
	MFilename whatName = MFilename::getSaveFileName(MFilename::Tile, this);

	if(whatName.isValid()){
		whatFileName = whatName;
		commit();
		what->save(whatFileName);
	}
}

void TileEdit::on_action_Open_activated(){
	MFilename fileToOpen = MFilename::getOpenFileName(MFilename::Tile, this);

	delete what;
	what = new MTile();

	what->load(fileToOpen);

	whatFileName = fileToOpen.forDisk();
	revert();
}

